require "config"
require "util"

module CacheStat
  module_function
  #
  # get status list
  #
  def list()
    Util.printtrace() ## debug

    stat = {}
    IO.foreach($conf["stat"]) {|line|
      line.chomp!
      s = line.split(/<>/)
      stat[s[0]] = {
        "stamp"   => s[1].to_i,
        "records" => s[2].to_i,
        "size"    => s[3].to_i,
      }
    }
    return stat
  rescue Errno::ENOENT, Errno::EACCES
    return {}
  end

  #
  # sync hash to file
  #
  def sync(stat)
    Util.printtrace(stat) ## debug

    Util.lock()
    File.open($conf["stat"], "w") {|f|
      stat.each {|key,val|
        next if key.nil? || (key == "")
        stamp   = val["stamp"  ] || "0"
        records = val["records"] || "0"
        size    = val["size"   ] || "0"
        f << "#{key.to_s}<>#{stamp.to_s}<>#{records.to_s}<>#{size.to_s}\n"
      }
    }
  rescue Errno::ENOENT, Errno::EACCES
    raise "#{$conf["stat"]}: open failed. #{$!}"
  ensure
    Util.unlock()
  end

  #
  # update latest status
  #
  def update()
    Util.printtrace() ## debug

    names = Dir.glob("#{$conf["datadir"]}/*.dat")
    stat = Hash.new

    Util.lock();

    names.each {|name|
      tmp = {}

      data = (IO.readlines(name) rescue [])
      if data.size > 0 then
        tmp["stamp"] = data[-1].split(/<>/)[0]
      else
        tmp["stamp"] = 0
      end

      tmp["records"] = data.size
      tmp["size"]    = FileTest.size?(name) || 0

      name.sub!(/#{$conf["datadir"]}\//, '')
      name.sub!(/\.dat$/,'')
      stat[name] = tmp
    }
    Util.unlock()
    sync(stat)
  end
end
