#!/usr/bin/ruby -Ku

require "config"
require "node"
require "nodelist"
require "cache"
require "cachestat"
require "util"


#
# save status
#
def save(time)
  Util.lock()
  File.open($conf["client"], "w") {|f|
    f << "#{time["ping"]}\n#{time["sync"]}\n#{time["init"]}\n"
  }
  Util.unlock();
#rescue
#  raise "Failed to open #{$conf["client"]}. #{$!}"
end

#
# main
#
def main
  $conf["debugid"] = "clnt"
  File.umask($conf["umask"])

  time = {}
  if ENV["REMOTE_ADDR"] !~ /#{$conf["adminAddr"]}/ then
    print "You are not the administrator.\n"
    exit
  end

  begin
    File.open($conf["client"], "r") {|f|
      time["ping"] = f.gets.chomp.to_i
      time["sync"] = f.gets.chomp.to_i
      time["init"] = f.gets.chomp.to_i
    }
  rescue
    time["ping"], time["sync"], time["init"] = 0, 0, 0
  end

  Cache.newFiles()

  nowsec = Time.new.to_i

  if (nowsec - time["ping"]) >= ($conf["pingfrec"] * 60) then
    time["ping"] = nowsec
    save(time)
    NodeList.pingall()
  end

  nodes = NodeList.all()

  if nodes.size == 0 then
    time["init"] = nowsec
    save(time)
    NodeList.init()

    time["sync"] = nowsec
    save(time)
    CacheStat.update()
    Cache.sync()
  end

  nc = NodeList.count()
  if (nowsec - time["init"]) >= ($conf["initfrec"] * nc * 60) then
    time["init"] = nowsec
    save(time)  
    NodeList.init()
  end

  if (nowsec - time["sync"]) >= ($conf["syncfrec"] * 60) then
    nodes.each {|node|
      NodeList.join(node)
    }
    time["sync"] = nowsec
    save(time)
    CacheStat.update()
    Cache.sync()
  end
end


  # child process
  Signal.trap("TERM", "DEFAULT")
  Signal.trap("HUP",  "DEFAULT")
  Signal.trap("CHLD", "IGNORE" )
  sleep(5)

  loop {
    pid = fork {
      # grandchild process
      ENV["QUERY_STRING"]   = ""
      ENV["PATH_INFO"]      = ""
      ENV["REQUEST_METHOD"] = "GET"
      ENV["REMOTE_ADDR"]    = "127.0.0.1"
      ENV["SCRIPT_NAME"]    = "/client.rb"

      Signal.trap("CHLD", "DEFAULT")
      Signal.trap("ALRM") { raise "client.cgi timeout"}
      Util.alarm($conf["timeout_client"])
      main()
      exit
    }

    if pid < 0 then
      dprint "client process: fork failed."
      sleep($conf["clientfrec"] * 60)

    else
      # child process
      sleep(10)

      # normal random by Box-Muller Transformation
      include Math
      r1, r2 = rand, rand
      normal = Math.sqrt(-2 * Math.log(r1)) * Math.sin(2 * 3.141592653 * r2);

      t = ($conf["clientfrec"] * 60) + ($conf["timeError"] * normal).to_i
      t = ($conf["clientfrec"] * 60) if t <= 0
      sleep(t)
    end
  } # --> loop
