require "config"
require "cache"
require "cachestat"
require "gateway"
require "message"
require "util"

class ListCGI < Gateway
  def initialize
    super
    $conf["debugid"] = "list" # debug
    @types = %w(list thread note)
  end

  #
  # main routine
  #
  def execute(path, input)
    Util.printtrace() ## debug

    File.umask($conf["umask"])

    arg  = args(input)
    cmd    = arg["cmd"   ] || ""
    file   = arg["file"  ] || ""
    record = arg["record"] || ""

    if (cmd == "new") && (file =~ /^list_[0-9A-F]+$/) then
      newElement(file, arg)
      print302("#{@root}list.cgi/" + str_encode(file_decode(arg["file"])))

    elsif cmd == "check" then
      checkSign(arg)

    elsif cmd == "delete" then
      checkAdmin()
      deleteDialog(arg)

    elsif cmd == "xdelete" then
      checkAdmin()
      if arg["file"].nil? then
        print404()
        exit
      elsif arg["stamp"] && arg["id"] then
        deleteRecord(arg["file"], arg["stamp"], arg["id"], arg)
      else
        deleteFile(arg["file"])
      end

    elsif path == "" then
      printList($conf["menufile"])

    elsif path !~ /\//  then
      printList(path)

    else
      print404()
      exit
    end
  end

  #
  # print list
  #
  def printList(list)
    Util.printtrace(list) ## debug

    list = escape(list)
    file_list = file_encode("list", list)
    touch(file_list)

    f = File.open("#{@datadir}/#{file_list}.dat", "r")
    printHeader(list)
    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}'>" <<
             "<p><input type='hidden' name='cmd' value='delete' />" <<
             "<input type='hidden' name='file' value='#{file_list}' /></p><ul>\n"

    bufs = []

    stat = CacheStat.list()
    count = 0

    f.each_line {|line|
      count += 1
      rec = rec(line) # Hash
      next if rec.size <= 2
      rec["body"] = "" if rec["body"].nil?
      stamp = rec["stamp"]

      if rec["type"] then
        file_child = file_encode(rec["type"], rec["link"])

        records = 0
        size = 0
        if FileTest.size?("#{@datadir}/#{file_child}.dat") then
          stamp   =  stat[file_child]["stamp"]
          records =  stat[file_child]["records"]
          size    = (stat[file_child]["size"]/1024/1024*10).to_i / 10
        end

        records = "?" if records <= 0
        str = xlocaltime(stamp)
        child = str_encode(rec["link"])

        anchor = rec["link"]
        if rec["type"] =~ /^(list|thread|note)$/ then
          anchor = "<a href='#{@root}#{rec["type"]}.cgi/#{child}'>#{rec["link"]}</a>"
        end

        buf = "<li><input type='radio' name='record' value='#{rec["stamp"]}/#{rec["id"]}' " +
              "tabindex='1' accesskey='s' />" +
              "#{str}: #{anchor}" +
              "(#{@message[rec["type"]]}/#{records}/#{size}#{@message["mb"]})#{rec["body"]}</li>\n"

      elsif rec["remove_stamp"] then
        stamp = rec["remove_stamp"]
        str = xlocaltime(stamp)
        check = ""
        rec["name"] = "" if rec["name"].nil?
        if rec["pubkey"] then
          check = "<a href='#{ENV["SCRIPT_NAME"]}?cmd=check&amp;" +
                  "file=" + file_encode("list", list) + "&amp;" +
                  "stamp=#{rec["stamp"]}&amp;id=#{rec["id"]}'>" +
                  Signature.pubkey2trip(rec["pubkey"]) + "</a>"
        end
        buf = "<li><input type='radio' name='record' value='#{rec["stamp"]}/#{rec["id"]}' " +
              "tabindex='1' accesskey='s' />" +
              "#{str}: #{rec["name"]} #{check} #{@message["remove"]}: #{rec["body"]}</li>\n"
      end

      bufs << [stamp.to_i, buf]
    }
    f.close

    bufs.sort! {|a,b| b[0] <=> a[0] }
    bufs.each  {|x|   @body << x[1] }

    @body << "</ul><p><input type='submit' value='#{@message["del_record"]}' tabindex='2' accesskey='d' />" <<
             "<input type='hidden' name='mode' value='list' /></p></form>"

    filesize = (FileTest.size("#{@datadir}/#{file_list}.dat")/1024/1024*10).to_i / 10
    checked = (count > 0) ?  "checked='checked'" : ""
    if filesize <= $conf["filelimit"] then
      newElementForm( {"file"=>file_list, "checked"=>checked, "dopost"=>1} )
    end

    @body << "<form action='#{ENV["SCRIPT_NAME"]}'>" <<
          "<p><input type='submit' value='#{@message["del_file"]}' tabindex='6' accesskey='d' />" <<
          "<input type='hidden' name='cmd' value='delete' />" <<
          "<input type='hidden' name='file' value='#{file_list}' />" <<
          " #{filesize}#{@message["mb"]}</p></form></body></html>\n"

  rescue Errno::ENOENT, Errno::EACCES
    raise "#{@datadir}/#{file_list}.dat: failed to open. #{$!}#{$@}"
  end
end
