require "config"
require "cache"
require "cachestat"
require "gateway"
require "message"
require "util"

class NoteCGI < Gateway
  def initialize
    super
    $conf["debugid"] = "note" # debug
  end

  #
  # main routine
  #
  def execute(path, input)
    Util.printtrace() ## debug

    File.umask($conf["umask"])

    arg    = args(input)
    cmd    = arg["cmd"   ] || ""
    file   = arg["file"  ] || ""
    record = arg["record"] || ""

    if (cmd == "post") && (file =~ /^note_[0-9A-F]+/) then
      id = post(file, arg)
      print302("#{@root}note.cgi/" + str_encode(file_decode(file)))

    elsif (cmd == "add") && (file =~ /^note_[0-9A-F]+/) then
      margeRecord(file, arg)
      print302("#{@root}note.cgi/" + str_encode(file_decode(file)))

    elsif cmd == "delete" then
      checkAdmin()
      deleteDialog(arg)

    elsif cmd == "xdelete"
      checkAdmin()
      if arg["file"].nil? then
        print404()
        exit
      elsif arg["stamp"] && arg["id"] then
        deleteRecord(arg["file"], arg["stamp"], arg["id"], arg)
      else
        deleteFile(arg["file"])
      end

    elsif (path != "") && (path !~ /\//) then
      if arg["mode"].nil? || (arg["mode"] == "") then
        printNote(path)
      elsif arg["mode"] == "edit" then
        editDialog(path, "note")
      elsif arg["mode"] == "history" then
        printHistory(path, "note")
      else
        printNote(path)
      end

    else
      print404()
      exit
    end
  end

  #
  # print note
  #
  def printNote(note)
    Util.printtrace(note) ## debug

    note = escape(note)
    file_note = file_encode("note", note)
    touch(file_note)

    printHeader(note)

    buf, conflict = readNote(file_note)
    hrec = rec(buf)
    hrec["body"] = "" if hrec["body"].nil?

    body = html_format(hrec["body"])
    @body << "\n<p>#{body}</p>\n"
    @body << "<p><em>#{@message["conflict"]}</em></p>" if conflict > 0

    filesize = (FileTest.size("#{@datadir}/#{file_note}.dat")/1024/1024*10).to_i / 10
    if filesize <= $conf["filelimit"] then
      @body << "<form method='post' action='#{ENV["SCRIPT_NAME"]}'><p>" <<
               "<input type='hidden' name='cmd' value='add' />" <<
               "<input type='hidden' name='file' value='#{file_note}' />" <<
               "<input type='submit' value='#{@message["add"]}' name='submit' tabindex='1' accesskey='w' />" <<
               " <input type='checkbox' value='dopost' name='dopost'" <<
               " checked='checked' tabindex='2' accesskey='s' />" <<
               "#{@message["send"]}" <<
               " <input type='checkbox' value='error' name='error'" <<
               " checked='checked' tabindex='2' accesskey='e' />" <<
               "#{@message["error"]}<br />" <<
               "<textarea rows='5' cols='70' name='message' tabindex='8' accesskey='c'></textarea>" <<
               "<br /><a href='#{ENV["SCRIPT_NAME"]}/" << str_encode(note) << "?mode=edit'>#{@message["edit"]}</a>" <<
               " | <a href='#{ENV["SCRIPT_NAME"]}/" << str_encode(note) << "?mode=history'>#{@message["history"]}</a>" <<
               "</p></form>"
    else
      @body << "<p><a href='#{ENV["SCRIPT_NAME"]}/note/" << str_encode(note) << "?mode=history'>#{@message["history"]}</a></p>"
    end
    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}'>" <<
             "<p><input type='submit' value='#{@message["del_file"]}' tabindex='9' accesskey='d' />" <<
             "<input type='hidden' name='cmd' value='delete' />" <<
             "<input type='hidden' name='file' value='#{file_note}' />" <<
             " #{filesize}#{@message["mb"]}</p></form></body></html>\n"
  end

  #
  # edit form
  #
  def editForm(file, rec)
    Util.printtrace(file, rec) ## debug

    body = rec["body"] || ""
    body.gsub!(/<br>/, "\n")
    @body << "<form method='post' action='#{ENV["SCRIPT_NAME"]}'><p>" <<
             "<input type='hidden' name='cmd' value='post' />" <<
             "<input type='hidden' name='file' value='#{file}' />" <<
             "<input type='submit' value='#{@message["post"]}' name='submit' tabindex='1' accesskey='w' />" <<
             " <input type='checkbox' value='dopost' name='dopost'" <<
             " checked='checked' tabindex='2' accesskey='s' />" <<
             "#{@message["send"]}" <<
             " <input type='checkbox' value='error' name='error'" <<
             " checked='checked' tabindex='2' accesskey='e' />" <<
             "#{@message["error"]}<br />" <<
             "<textarea rows='20' cols='75' name='body' tabindex='8' accesskey='c'>#{body}</textarea>" <<
             "<input type='hidden' value='#{rec["stamp"]}' name='base_stamp' />" <<
             "<input type='hidden' value='#{rec["id"]}' name='base_id' />" <<
             "</p></form>"
  end

  #
  # print note history
  #
  def printHistory(name, type)
    Util.printtrace(name, type) ## debug

    name = escape(name)
    file = file_encode(type, name)
    touch(file)

    lines = IO.readlines("#{@datadir}/#{file}.dat")
    printHeader(name)
    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}'>" <<
             "<p><input type='hidden' name='cmd' value='delete' />" <<
             "<p><input type='hidden' name='file' value='#{file}' /></p><dl>\n"

    ref = {}
    last = ""
    lines.each {|line|
      rec = rec(line)
      next if rec.size <= 2

      key = rec["stamp"] + "_" + rec["id"]

      ref[key] = 0
      if rec["base_stamp"].nil? then
        ref[key] = 1
        next
      else
        key = rec["base_stamp"] + "_" + rec["base_id"]
        ref[key] += 1 if ref[key]
      end
      last = line
    }

    if last != "" then
      rec = rec(last)
      ref["#{rec["stamp"]}_#{rec["id"]}"] = 1
    end

    lines.each {|line|
      rec = rec(line)
      next if rec.size <= 2

      xstamp = xlocaltime(rec["stamp"])
      rec["body"] = "" if rec["body"].nil?
      rec["body"] = html_format(rec["body"])

      base = ""
      if rec["base_id"] && (rec["base_id"] != "") then
        base = rec["base_id"][0, 8] # substr
        base.gsub!(/\"/, "&quot;")
        base = "(<a href=\"#r#{base}\">&gt;&gt;#{base}</a>)" if base
      end

      remove = ""
      if rec["remove_id"] && rec["remove_stamp"] then
        xid = rec["remove_id"][0, 8]
        remove = "[[#{@message["remove"]}: <a href='#r#{xid}'>#{xid}</a>]]<br />"
      end
      xid = rec["id"][0, 8]
      conflict = (ref["#{rec["stamp"]}_#{rec["id"]}"]) > 0 ? "" : " <em>#{@message["conflict"]}</em>"
      @body << "<dt id='r#{xid}'><input type='radio' name='record' value='#{rec["stamp"]}/#{rec["id"]}' " <<
               "tabindex='1' accesskey='s' />" <<
               "#{xid} #{xstamp} #{base}#{conflict}</dt><dd>#{rec["body"]}<br />#{remove}<br /></dd>\n"
    }

    @body << "</dl><p><input type='submit' value='#{@message["del_record"]}' tabindex='2' accesskey='d' />" <<
             "<input type='hidden' name='mode' value='#{type}' /></p></form>"

    buf, = readNote(file)
    rec = rec(buf)
    editForm(file, rec)
    filesize = (FileTest.size?("#{@datadir}/#{file}.dat")/1024/1024*10).to_i/10
    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}'>" <<
             "<p><input type='submit' value='#{@message["del_file"]}' tabindex='9' accesskey='d' />" <<
             "<input type='hidden' name='cmd' value='delete' />" <<
             "<input type='hidden' name='file' value='#{file}' />" <<
             " #{filesize}#{@message["mb"]}</p></form></body></html>\n"
  end

  #
  # edit dialog
  #
  def editDialog(name, type)
    Util.printtrace(name, type) ## debug

    name = escape(name)
    file = file_encode(type, name)
    touch(file)

    buf, conflict = readNote(file)
    rec = rec(buf)
    rec["body"] = "" if rec["body"].nil?
    xbody = html_format(rec["body"])

    printHeader(name)
    @body << "\n<p>#{xbody}</p>\n"
    @body << "<p><em>#{@message["conflict"]}</em></p>" if conflict > 0
    editForm(file, rec)

    filesize = (FileTest.size("#{@datadir}/#{file}.dat")/1024/1024*10).to_i / 10
    @body << "<form method='get' action='#{ENV["SCRIPT_NAME"]}'>" <<
             "<p><input type='submit' value='#{@message["del_file"]}' tabindex='9' accesskey='d' />" <<
             "<input type='hidden' name='file' value='#{file}' />" <<
             "<input type='hidden' name='cmd' value='delete' />" <<
             " #{filesize}#{@message["mb"]}</p></form></body></html>\n"
  end
end
