
require "config"
require "util"

class Node

  def initialize(host, port = nil, path = nil)
    Util.printtrace(host, port, path) ## debug
    @host, @port, @path = '', '', ''

    if host && port && path then
      @host, @port, @path = host, port, path
    elsif host then
      host =~ /([^:]+):(\d+)(.*)/
      @host, @port, @path = $1, $2, $3
    else
      return nil
    end

    @path.gsub!(/\+/, "/")
    @path = "/" if @path == ""
  end

  attr_reader("host", "port", "path")  # property xxx read Fxxx;

  #
  # connect other node
  # args: message
  def talk(message)
    Util.printtrace() ## debug

    message.sub!(/^[\/?]/, '')

    path = @path.dup
    if path =~ /[\/?]$/ then
      path += message
    else
      path += "/#{message}"
    end

    Util.debug("** talk() http://#{@host}:#{@port}#{path}")

    buf = Util.wget("http://#{@host}:#{@port}#{path}")

    while (buf.size > 0) && (buf[0] !~ /^\s*\r\n$/) do
      s = buf.shift
      gzip = true if s =~ /Content-Encoding:\s*gzip/i
    end;

    return [] if buf == []
    buf.shift
    return [] if buf == []

    Util.debug("** talk() ==> ", buf)

#    if gzip then
#      require "extern"
#      buf = Extern::xgunzip(buf)
#    end

    buf.pop if buf[-1] !~ /\n$/
    buf.collect! {|s| s.gsub(/[\n\r]/, '') }

    return buf
  end

  #
  # to string
  #
  def toString()
    return "#{@host}:#{@port}#{@path}"
  end

  #
  # to +string
  #
  def toXString()
    path = @path.sub(/\//, "+")
    return "#{@host}:#{@port}#{path}"
  end

  #
  # ping node
  #
  def ping()
    Util.printtrace() ## debug
    buf = talk("/ping")
    return (buf.size >= 1) && (buf[0] == "PONG")
  end

  #
  # connect
  #
  def join()
    Util.printtrace() ## debug
    path = $conf["path"].gsub(/\//, "+")
    return talk("/join/#{ENV["SERVER_NAME"]}:#{ENV["SERVER_PORT"]}#{path}")
  end

  #
  # unconnect
  #
  def bye()
    Util.printtrace() ## debug
    path = $conf["path"].gsub(/\//, "+")
    buf = talk("/bye/:#{ENV["SERVER_PORT"]}#{path}")
    return (buf.size > 0) && (buf[0] == "BYEBYE")
  end
end
